/***********************************************************************
 * File: EUC_code_counter.h
 * 	(Extracted from
 *	   "$Id: EUC_code_counter.h,v 1.2 2009/05/23 12:47:52 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header defines prototyping for the EUC-JP character counter,
 *  and getting sub-string of the EUC-JP encode string.
 *
 * Copyright (C) 2005-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/


#ifndef _EUC_CODE_COUNTER_H_ /* { */
#define _EUC_CODE_COUNTER_H_
#ifdef __cplusplus
extern "C" {
#endif

/* including the system definitions */
#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <limits.h>


/* get the size of EUC-JP string buffer
 * (we must specify number of the single characters)
 */
#define GET_EUC_JP_STR_BUF_SZ(SINGLE_CHAR_N)	(2 * (SINGLE_CHAR_N))

/* count up the EUC-JP characters in a string */
/* definition of type for "count_euc_code_characters()" output */
typedef struct euc_code_character_class_counts_t {
	size_t	length_of_bytes
			, count_of_new_lines
			, count_of_controls
			, count_of_ASCII
			, count_of_JIS_X_0208_1983
			, count_of_JIS_X_0201_1976
			, count_of_JIS_X_0212_1990
			, count_of_UNKNOWN
			;
} EUC_code_character_class_counts;

/* definitions of ASCII control codes */
#define ASCII_CNTL_NUL	'\0' 	/* null				*/
#define ASCII_CNTL_SOH	'\x1'	/* start of header		*/
#define ASCII_CNTL_STX	'\x2'	/* start of text		*/
#define ASCII_CNTL_ETX	'\x3'	/* end of text			*/
#define ASCII_CNTL_EOT	'\x4'	/* end of transfer		*/
#define ASCII_CNTL_ENQ	'\x5'	/* enquirer			*/
#define ASCII_CNTL_ACK	'\x6'	/* acknowledgment		*/
#define ASCII_CNTL_BEL	'\a'	/* bell				*/
#define ASCII_CNTL_BS	'\b'	/* back space			*/
#define ASCII_CNTL_HT	'\t'	/* horizontal tab		*/
#define ASCII_CNTL_LF	'\n'	/* line feed			*/
#define ASCII_CNTL_VT	'\v'	/* vertical tab			*/
#define ASCII_CNTL_FF	'\f'	/* form feed			*/
#define ASCII_CNTL_CR	'\r'	/* carriage return		*/
#define ASCII_CNTL_SO	'\xe'	/* shift out			*/
#define ASCII_CNTL_SI	'\xf'	/* shift out			*/
#define ASCII_CNTL_DLE	'\x10'	/* data link escape		*/
#define ASCII_CNTL_DC1	'\x11'	/* device control #1		*/
#define ASCII_CNTL_DC2	'\x12'	/* device control #2		*/
#define ASCII_CNTL_DC3	'\x13'	/* device control #3		*/
#define ASCII_CNTL_DC4	'\x14'	/* device control #4		*/
#define ASCII_CNTL_NAK	'\x15'	/* not acknowledgment		*/
#define ASCII_CNTL_SYN	'\x16'	/* synchronize			*/
#define ASCII_CNTL_ETB	'\x17'	/* end of transfer blocks	*/
#define ASCII_CNTL_CAN	'\x18'	/* cancel			*/
#define ASCII_CNTL_EM	'\x19'	/* end of medium		*/
#define ASCII_CNTL_SUB	'\x1a'	/* substitute			*/
#define ASCII_CNTL_ESC	'\x1b'	/* escape			*/
#define ASCII_CNTL_FS	'\x1c'	/* file separator		*/
#define ASCII_CNTL_GS	'\x1d'	/* group separator		*/
#define ASCII_CNTL_RS	'\x1e'	/* record separator		*/
#define ASCII_CNTL_US	'\x1f'	/* unit separator		*/
#define ASCII_CNTL_DEL	'\x7f'	/* delete			*/


extern size_t count_euc_code_characters(
								  unsigned char *str
								, size_t	str_sz
								, EUC_code_character_class_counts *counts
								);

/* count up the characters those are converted from EUC-JP to SJIS */
#define count_euc_characters_as_sjis(STR,STR_SZ,COUNTS)	((size_t)(	\
	((count_euc_code_characters((STR), (STR_SZ), (COUNTS)) == 0)	\
	|| errno != 0)?	0												\
	:				(2 * ((COUNTS)->count_of_JIS_X_0208_1983		\
						+ (COUNTS)->count_of_JIS_X_0212_1990)		\
						+ (COUNTS)->count_of_new_lines				\
						+ (COUNTS)->count_of_JIS_X_0201_1976		\
						+ (COUNTS)->count_of_ASCII					\
						)											\
	))


/* get sub-string address of a EUC-JP string, and output size of sub-string
 */
extern char *substr_of_euc_code_string(unsigned char *str
									, size_t		str_sz
									, size_t		nth_char
									, size_t		n_chars
									, size_t		w_chars
									, char			*delimits
									, size_t		(*metrics)(
														  unsigned char *str
														, size_t	sz
														, void		*usrdata
														)
									, void			*usrdata
									, size_t		*size
									, size_t		*width
									);


/* type definition of the trimming direction */
typedef enum trim_EUCJP_spaces_direction_t {
	  Trim_EUCJP_spaces_LEFT		= 0x01
	, Trim_EUCJP_spaces_RIGHT		= 0x02
	, Trim_ASCII_all_space_codes	= 0x04
}	Trim_EUCJP_spaces_direction_t;


/* trimming the EUC-JP wide character spaces and ASCII spaces */
extern char *trim_EUCJP_spaces(Trim_EUCJP_spaces_direction_t	direct
								, char							*str
								);

/* copy a string that encoded EUC-JP */
extern char *strcpy_EUCJP(unsigned char *src, char *dst, size_t dst_sz);

#ifdef __cplusplus
}
#endif
#endif /* } (!defined(_EUC_CODE_COUNTER_H_) */
