/***********************************************************************
 * File: unit_test.c
 * 	(Extracted from
 *	   "$Id: unit_test.c,v 1.2 2009/05/22 16:57:20 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source code implements the unit test for sql_quoting().
 *
 * Copyright (C) 2006-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

/* include system definitions */
#include <stdlib.h>
#include <stdio.h>
extern int snprintf(char *str, size_t size, const char *format, ...);

/* include definitions for sql_quoting() API */
#include "sql_quoting.h"

int main(int ac, char **av)
{
	char					*enclosed[] = { "", "'", "\"" }
							, escapes[] = { '\0', '\\', '\t', '@' }
							, val[15]
							, src[sizeof(val)]
							, dst[(sizeof(src) - 1) * 2 + 3]
							, *result = NULL, *errmsg = NULL
							, **sv_av = av
							;
	SQL_quoting_qualifier_t	w, e, pat, enc, sta, vert
							, qualifier = SQL_quoting_default
							;
	size_t					en, qn, dst_sz, escn;
	int						sv_ac = ac, sv_errno;
	size_t					gprof_cnt, gprof_iter_max;

	for (gprof_iter_max = (size_t)((ac == 2 && strcmp(av[1], "-P") == 0)?
																	10: 1
									)
		, gprof_cnt = 0
	; gprof_cnt < gprof_iter_max
	; ++gprof_cnt
	) {	/* iterate the unit test for gprof profiling... */
	ac = sv_ac; av = sv_av;
	printf("{{{ test for sql_quoting() starts...\n\n"
		); (void)fflush(stdout);

	strcpy(val, "100%_|ove\\" SQL_quoting_CONTROL_CHARACTERS);

	for (en = 0; en < sizeof(enclosed) / sizeof(enclosed[0]); ++en)
	for (qn = 0; qn < sizeof(enclosed) / sizeof(enclosed[0]); ++qn)
	for (vert = (SQL_quoting_qualifier_t)0x0
	; vert <= SQL_quoting_escape_vertical_bar
	; vert = (SQL_quoting_qualifier_t)(vert
				+ SQL_quoting_escape_vertical_bar)
	)
	for (sta = (SQL_quoting_qualifier_t)0x0
	; sta <= SQL_quoting_do_not_escape_quots
	; sta = (SQL_quoting_qualifier_t)(sta
				+ SQL_quoting_do_not_escape_quots)
	)
	for (enc = (SQL_quoting_qualifier_t)0x0
	; enc <= SQL_quoting_enclose_result
	; enc = (SQL_quoting_qualifier_t)(enc
				+ SQL_quoting_enclose_result)
	)
	for (pat = (SQL_quoting_qualifier_t)0x0
	; pat <= SQL_quoting_escape_pattern_codes
	; pat = (SQL_quoting_qualifier_t)(pat
				+ SQL_quoting_escape_pattern_codes)
	)
	for (e = (SQL_quoting_qualifier_t)0x0
	; e <= SQL_quoting_escape_control_codes
	; e = (SQL_quoting_qualifier_t)(e
				+ SQL_quoting_escape_control_codes)
	)
	for (w = (SQL_quoting_qualifier_t)0x0
	; w <= SQL_quoting_with_backslash
	; w = (SQL_quoting_qualifier_t)(w + SQL_quoting_with_backslash)
	)
	for (dst_sz = sizeof(dst)
	; (dst_sz >= sizeof(src)) && dst_sz <= sizeof(dst)
	; dst_sz -= sizeof(src))
	for (escn = 0
	; escn < sizeof(escapes) / sizeof(escapes[0])
	; ++escn) {
		errno	= 0;
		(void)snprintf(src, sizeof(src), "%s%s%s"
				, enclosed[en], val, enclosed[en]
				);
		qualifier	= (SQL_quoting_qualifier_t)(
					w | e | pat | enc | sta
				);
		memset(dst, (int)'@', sizeof(dst) - 1);
		dst[sizeof(dst) - 1]	= '\0';
		result	= sql_quoting(src
					, qualifier
					, *(enclosed[qn])
					, escapes[escn]
					, dst
					, dst_sz
					);
		sv_errno	= errno;
		errmsg	= strerror(sv_errno);
		printf("sql_quoting(src@%p [%s] (len = %lu), "
			"qualifier = 0x%x, "
			"quote_char = '%c', escape_code = 0x%x, o"
			"dst@%p, "
			"dst_sz = %lu)\n"
			"...returns %p [%s] (len = %lu)"
			"(errno = %d [%s]).\n\n"
			,	src, src, (unsigned long)strlen(src)
			,	qualifier
			,	*(enclosed[qn]), (unsigned)escapes[escn]
			,	dst
			,	(unsigned long)dst_sz
			,	result, ((result == NULL)? "": result)
			,		(unsigned long)strlen(dst)
			,	sv_errno, errmsg
			); (void)fflush(stdout);
	}

	printf("}}} test for sql_quoting() finished!\n"
		); (void)fflush(stdout);
	} /* iterate the unit test for gprof profiling... */

	exit(0);
}
